/*
 * Copyright (C) 2002 Robert Ernst <robert.ernst@linux-solutions.at>
 *
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPL included in the
 * packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See COPYING for GPL licensing information.
 *
 */

#ifndef __BOARD_HEADER__
#define __BOARD_HEADER__

#include <qwidget.h>
#include <qpixmap.h>

class BoardWidget : public QWidget
{
    Q_OBJECT

public:
    BoardWidget(QWidget *parent, const char *name = 0, WFlags f = 0);
    ~BoardWidget();

private:
    QPixmap *m_pixmap;
    QPixmap m_board;
    QPixmap m_pieces;
    int m_player_color;
    int m_level;
    int m_animations;
    int m_moving;
    int m_moving_x;
    int m_moving_y;
    int m_moving_piece;
    enum {
	None,
	ComputerSet,
	ComputerMove,
	ComputerTake,
	PlayerSet,
	PlayerMoveStart,
	PlayerMoveEnd,
	PlayerTake,
    } m_state;

public slots:
    void updatePlayerColor(int player_color);
    void updateLevel(int level);
    void updateAnimations(int animations);
    void start(void);
    void undo(void);
    void redo(void);

protected:
    void paintEvent(QPaintEvent *event);
    void resizeEvent(QResizeEvent *event);
    void mouseReleaseEvent(QMouseEvent *event);

private:
    void animateMove(int start, int end, int piece);
    void makeMove(void);
    void computerMove(void);
    void gameOver(void);
};

#endif // __BOARD_HEADER__

