#include <assert.h>
#include "tree.hh"
#include "eval.hh"
#include "board.hh"
#include "gnmm.hh"

SearchAlgo algo;
CountEval eval_cnt;
Board board;
Move current_move;

int BoardMapData[7][7] =
{
  { 1, 2, 2, 3, 2, 2, 4},
  { 5, 1, 2, 6, 2, 4, 5},
  { 5, 5, 1, 7, 4, 5, 5},
  { 8, 6, 9,12, 8, 6, 9},
  { 5, 5,10, 3,11, 5, 5},
  { 5,10, 2, 6, 2,11, 5},
  {10, 2, 2, 7, 2, 2,11}
};

int validplace[7][7] =
{
  { 1,0,0,1,0,0,1 },
  { 0,1,0,1,0,1,0 },
  { 0,0,1,1,1,0,0 },
  { 1,1,1,0,1,1,1 },
  { 0,0,1,1,1,0,0 },
  { 0,1,0,1,0,1,0 },
  { 1,0,0,1,0,0,1 }
};

int coord2move[7][7] =
{
  { 0, 0, 0, 1, 0 ,0,2 },
  { 0, 8, 0, 9, 0,10,0 },
  { 0, 0,16,17,18, 0,0 },
  { 7,15,23, 0,19,11,3 },
  { 0, 0,22,21,20, 0,0 },
  { 0,14, 0,13, 0,12,0 },
  { 6, 0, 0, 5, 0, 0,4 }
};

int move2coord[24][2] =
{
  { 0,0 },
  { 0,3 },
  { 0,6 },
  { 3,6 },
  { 6,6 },
  { 6,3 },
  { 6,0 },
  { 3,0 },

  { 1,1 },
  { 1,3 },
  { 1,5 },
  { 3,5 },
  { 5,5 },
  { 5,3 },
  { 5,1 },
  { 3,1 },

  { 2,2 },
  { 2,3 },
  { 2,4 },
  { 3,4 },
  { 4,4 },
  { 4,3 },
  { 4,2 },
  { 3,2 }
};

int history_size;
int history_length;
int history_pos;
Board* history;

Move next_valid[200];  // all next available moves
int n_next_valid;

const int nLevels = 5;
int level2nodes[nLevels] = { 500,  15000,  100000,  300000, 1000000 };

void push(const Board& board)
{
  if (history_size <= history_pos)
    {
      history_size = history_size+100;

      Board* new_history = new Board[history_size];
      for (int i=0;i<history_length;i++)
	new_history[i] = history[i];

      if (history) delete[] history;

      history = new_history;
    }

  assert(history_pos < history_size);

  history[history_pos] = board;
  history_pos++;
  history_length = history_pos;
}

void gen_valid_moves()
{
  n_next_valid = board.GenMoves(next_valid,200);
}

bool is_valid_move(const Move& m)
{
  for (int i=0;i<n_next_valid;i++)
    if (m.start == next_valid[i].start &&
	m.end   == next_valid[i].end   &&
	m.take  == next_valid[i].take)
      return true;

  return false;
}

bool valid_moving(const Move& m)
{
  for (int i=0;i<n_next_valid;i++)
    if (m.start == next_valid[i].start &&
	m.end   == next_valid[i].end)
      return true;

  return false;
}

void game_init()
{
  current_move.Reset();

  algo.SetEvaluator(eval_cnt);
  algo.Init();
  board.Init();

  algo.StopComputer();

  history_length = 0;
  history_pos = 0;
  push(board);

  gen_valid_moves();
}

void game_exit()
{
    algo.StopComputer();
}

